;*****************************************************
;******************** Definiciones *******************
;*****************************************************

	DELK=24
	DELAB=32

	.area rom (abs)
	ppi=0
	ppia=ppi	
	ppib=ppi+1	
	ppic=ppi+2	
	ppict=ppi+3	

;*****************************************************
;****************** Inicializacion *******************
;*****************************************************

	.org 0		;Redundante?
start:  
	di
	im #1
	ld a,#0b10000010 ;PPI modo 0, PA out, PB in, PC out
	out (ppict),a
	ld sp,#2048	; Pila al final de la RAM
	jp main

;***************************************************
; INT (de momento nada)
;***************************************************
	.org 0x38
	ei
	reti

;***************************************************
; NMI (power OFF)
;***************************************************
	.org 0x66
	halt


;***************************************************
;Barrido y exploracion del teclado (una sola pasada)
;***************************************************

scan:	push af
	push bc
	push de
	push hl

buc1:	in a,(ppib)
	bit #6,a
	jr nz,buc1
buc2:	in a,(ppib)
	bit #6,a
	jr z,buc2
	ld hl,#video
	ld de,#keymap
	ld b,#6
	jr ent

buc3:	in a,(ppib)
	bit #7,a
	jr nz,buc3
buc4:	in a,(ppib)
	bit #7,a
	jr z,buc4
ent:	ld a,(hl)
	out (ppia),a
	in a,(ppib)
	and #0xf
	ld (de),a
	inc hl
	inc de
	djnz buc3

	pop hl
	pop de
	pop bc
	pop af
	ret


;********************************************************
;PUTCH: copia el codigo ASCII de A al video.
;C = posicion dentro de pantalla (se incrementa.)
;********************************************************

putch:	push af
	push de
	push hl

	ld d,a
	ld a,c
	cp #6
	jr nc,exput
	ld a,d

	ld hl,#video
	ld e,c
	ld d,#0
	add hl,de
	ex de,hl	; DE=posicion caracter en pantalla

	ld hl,#asctbl
	sub #32
	add a,l
	ld l,a
	ld a,#0
	adc h
	ld h,a		; HL=posicion del codigo

	ld a,(hl)
	ex de,hl
	cp #0x80
	jr nz,sput1	; Caso del punto o coma
	dec hl
	dec c
	or (hl)

sput1:	ld (hl),a
	inc c

exput:	pop hl
	pop de
	pop af
	ret

;********************************************************
;Puts: imprime la cadena ASCIIZ apuntada por HL
;********************************************************

puts:	push af
	push hl
bputs:	ld a,(hl)
	or a
	jr z,fputs
	call putch
	inc hl
	jr bputs
fputs:	pop hl
	pop af
	ret

;********************************************************
;CLS: rellena la pantalla del caracter A
;********************************************************

cls:	push bc
	push de
	push hl
	ld c,#0
	call putch
	ld bc,#5
	ld de,#video+1
	ld hl,#video
	ldir
	pop hl
	pop de
	pop bc
	ret

;********************************************************
;Scrool 1 posicion a la izquierda
;********************************************************

scrool:	push bc		;Desplaza toda la pantalla una posicion
	push de		;a la izquierda
	push hl
	ld de,#video
	ld hl,#video+1
	ld bc,#5
	ldir
	pop hl
	pop de
	pop bc
	ret

;********************************************************
;Comprueba si hay pulsada alguna tecla.
;Devuelve Codigo en A y CF en uno si pulsada o
;CF en 0 y A con basura si no se pulsa nada 
;********************************************************

ktest:	push bc
	push de
	push hl

	ld hl,#keymap
	ld de,#keytbl
	ld b,#6
bkt1:	push bc
	ld b,#4
	ld a,(hl)
	inc hl
bkt2:	bit #0,a
	jr z,skt1
	ld a,(de)
	scf		;CF=1
	pop bc
	jr exkt
skt1:	inc de
	rrca
	djnz bkt2
	pop bc
	djnz bkt1
	or a		;CF=0

exkt:	pop hl
	pop de
	pop bc
	ret

;**********************************************************
; Espera que se pulse una tecla y devuelve codigo en A
; Mientras tanto se refresca pantalla
;**********************************************************
getch:	push bc
	ld c,#DELK
bget1:	call scan
	call ktest
	jr nc,bget1
	ld b,c
bget2:	call scan
	djnz bget2
	pop bc
	ret

;**********************************************************
;Imprime A en Hexadecimal (C=columna)
;**********************************************************

phex8:	push af
	push hl
	push af
	srl a
	srl a
	srl a
	srl a
	ld hl,#hextbl
	add a,l
	ld l,a
	ld a,#0
	adc h
	ld h,a
	ld a,(hl)
	call putch
	pop af
	and #0xf
        ld hl,#hextbl
        add a,l
        ld l,a
        ld a,#0
        adc h
        ld h,a
        ld a,(hl)
        call putch
	pop hl
	pop af
	ret

;**********************************************************
;Imprime HL en hexadecimal (C=columna)
;**********************************************************
phex16:	push af
	ld a,h
	call phex8
	ld a,l
	call phex8
	pop af
	ret

;**********************************************************
; Aade nibble hexadecimal a DE (queda copia de E en A)
;**********************************************************

shex:	call getch	;aade digito hexadecimal
	cp #0x10	;a A y DE. si bien CY=>1
	ret nc		;si no se pulsa nmero CY=>0
	push bc
	ld b,#4
bsh:	sla e
	rl d
	djnz bsh
	pop bc
	or e
	ld e,a
	scf
	ret

;**********************************************************
;Programa principal
;**********************************************************

main:
;======================================
;	interprete de commandos
;======================================
	
command:		; Comprobamos si ya se pulsa una tecla
	call scan	; y en ese caso se cargan con los valores
	call ktest	; por defecto algunas variables
	jr nc,comm
	ld hl,#about
	ld (vpf1),hl
	ld hl,#0
	ld (vpf2),hl
	ld (vpf3),hl
	ld (vpf4),hl
	ld hl,#free
	ld (dir),hl
	ld hl,#msg1
	ld c,#0
	call puts
bmain1:	call scan
	call ktest
	jr c,bmain1

comm:	ld c,#0		;Bucle principal
	ld hl,(dir)
	call phex16	;imprime direccin actual
	ld a,#'.
	call putch	;imprime "."
	ld a,(hl)
	call phex8	;imprime dato actual
	ld e,a
	call shex	;espera que se pulse algo
	jr nc,case
	ld (hl),a	;actualiza dato si digito hexa
	jr comm

case:	ld b,#8		;distribuye la ejecucion de comandos
	ld hl,#jptbl
bcase:	cp (hl)
	jr z,exe1
	inc hl
	inc hl
	inc hl
	djnz bcase
	jr comm
exe1:	ld de,#comm
	push de		;direccin de retorno
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	ex de,hl
	jp (hl)

jptbl:	.db 0x10	;tabla de codigos y saltos
	.dw decre
	.db 0x11
	.dw incre
	.db 0x12
	.dw goto
	.db 0x13
	.dw exec
	.db 0x14
	.dw pf1
	.db 0x15
	.dw pf2
	.db 0x16
	.dw pf3
	.db 0x17
	.dw pf4

incre:	ld hl,(dir)	;comando incr
	inc hl
	ld (dir),hl
	ret
decre:	ld hl,(dir)	;comando decr
	dec hl
	ld (dir),hl
	ret
exec:	ld hl,(dir)	;comando exe
	jp (hl)
pf1:	ld hl,(vpf1)	;funciones programables
	jp (hl)
pf2:	ld hl,(vpf2)
	jp (hl)
pf3:	ld hl,(vpf3)
	jp (hl)
pf4:	ld hl,(vpf4)
	jp (hl)
goto:	ld a,#'-	;comando go
	call cls
	ld de,(dir)
bgoto:	ld c,#1
	ex de,hl
	call phex16
	ex de,hl
	call shex
	jr c,bgoto
	ld (dir),de
	ret

;**********************************************************
;Rutinas utiles
;**********************************************************

about:	call scan	;Mensaje de Copyright
	call ktest
	jr c,about
	ld a,#32
	call cls
bab0:	ld hl,#msgab
bab1:	ld a,(hl)
	inc hl
	or a
	jr z,bab0
	call scrool
	ld c,#5
	call putch
	ld b,#DELAB
bab2:	call scan
	djnz bab2
	call ktest
	jr nc,bab1
bab3:	call scan
	call ktest
	jr c,bab3
	ret



;**********************************************************
;TABLAS de la ROM
;**********************************************************

;**********************************************************
;Tabla ASCII -> 7SEG
;**********************************************************
asctbl:	.db 0x00,0x02,0x22,0x00,0x00,0x00,0x00,0x20,0x39,0x0f,0x00,0x00 ;/* 32-43 */
	.db 0x80,0x40,0x80,0x52,0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x07 ;/* 44-55 */
	.db 0x7f,0x6f,0x00,0x00,0x46,0x41,0x70,0x53,0x3b ;/* 56-64 */
	.db 0x77,0x7c,0x39,0x5e,0x79,0x71,0x3d,0x76,0x06,0x1e,0x75,0x38,0x37
	.db 0x37,0x3f,0x73,0x67,0x31,0x6d,0x78,0x3e,0x3e,0x3e,0x76,0x6e,0x5b ;/* 65-90 */
	.db 0x39,0x64,0x0f,0x23,0x08,0x20 ;/* 91-96 */
	.db 0x77,0x7c,0x58,0x5e,0x79,0x71,0x3d,0x74,0x04,0x1e,0x75,0x38,0x37
	.db 0x54,0x5c,0x73,0x67,0x50,0x6d,0x78,0x1c,0x1c,0x3e,0x76,0x6e,0x5b ;/* 97-122 */
	.db 0x46,0x30,0x70,0x01,0x00 ;/* 123-127*/
;**********************************************************
;Tabla del teclado
;**********************************************************
keytbl:	.db 0x0a,0x0d,0x12,0x14	;Col 0
	.db 0x0b,0x0e,0x13,0x15	;Col 1
	.db 0x0c,0x0f,0x17,0x16	;Col 2
	.db 0x00,0x01,0x04,0x07	;Col 3
	.db 0x10,0x02,0x05,0x08	;Col 4
	.db 0x11,0x03,0x06,0x09	;Col 5
;**********************************************************
;Tabla Hex -> ASCII
;**********************************************************
hextbl:	.ascii "0123456789abcdef"
;**********************************************************
;Mensajes varios
;**********************************************************
msg1:	.ascii "def. . . ."
	.db 0
msgab:	.ascii "(c) Jesus Arias 1996 "
	.db 0

;*************************************************************
;*********************   RAM AREA ****************************
;*************************************************************
	.org 1024

vpf1:	.ds 2
vpf2:	.ds 2
vpf3:	.ds 2
vpf4:	.ds 2
dir:	.ds 2
video:	.ds 6
keymap:	.ds 6
free:
