/****************************************************************************

Emulador del hardware de la wopr

*****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <forms.h>
#include "Z80.h"

/************************* Descripcin de la memoria ************************/

#define	ROMEND	1023	/** ltima posicin de la ROM **/
#define RAMEND	2047	/** ltima posicin de la RAM **/
#define DIRMASK	2047	/** Mscara con el n de bit de direcciones usados */
#define	NOMEM	0	/** Accesos a zonas de no memoria **/

byte *MEM;
byte PPI[4]={0,0,0,0};
reg CPUREG;

byte col=0;
byte key[6]={0,0,0,0,0,0};
byte display[6];

void dodisplay(byte *);

void loadmem(char *fn,word offset)
{
	int fd;
	if ((fd=open(fn,O_RDONLY))==-1) {
		printf("Memory Image not found :%s\n",fn);
		exit(0);
	}
	read(fd,MEM+offset,ROMEND+1);
	close(fd);
}

void M_WRMEM(word dir,byte dato)
{
	dir&=DIRMASK;
	if (dir>ROMEND) MEM[dir]=dato;
}

byte M_RDMEM(word dir)
{
	dir&=DIRMASK;
	if (dir<=RAMEND) return MEM[dir]; else return NOMEM;
}

void PRG_ROM(word dir,byte dato)
{
	dir&=DIRMASK;
	MEM[dir]=dato;
}



extern void Draw7s(unsigned char dato, int pos);

void DoOut(byte dir,byte dato)
{
	dir&=3;
	PPI[dir]=dato;
}

#define NIN 24
extern int alflag; 

byte DoIn(byte dir)
{ 
	static int c=0,sl=0;

	dir&=3;

	if (dir==1) {
		c++; c%=NIN;
	
		/**** Oscilador de barrido **/
	
		PPI[1]=key[col];
		if (!col) PPI[1]|=0x40; else PPI[1]&=0xbf;
		if (c>=NIN/2) PPI[1]|=0x80; else PPI[1]&=0x7f;

		if (c==NIN/2) {	
			if (PPI[0]!=display[col]) {
				Draw7s(PPI[0],col);
				display[col]=PPI[0];
			}

			/******** Refresco XForms ********/
			if (alflag) {
				fl_check_forms();
				alflag=0;
			}

			col++; col%=6;

	/**** Detiene emulacin tras varios refrescos consecutivos ****/
			if (col==0) {
				sl++;
				if (sl==8) {
					sl=0;
					pause();
				}
			}
			
		}
	}
	return PPI[dir];
}


/*****************************************************************/
/***							       ***/

void usemsg()
{
	printf("\n\n-- W.O.P.R. --  (v2.1) Z80 prototype emulator by Jess Arias\n\n");
	printf("Syntaxis:\n");
	printf("wopr [options] [Hex_files]\n\n");
	printf("Options:\n");
	printf("-b 0xnnnn binary_file\tLoad image file at dir 0xnnnn\n");
	printf("-h\t\t\tShows this message\n\n");
	printf("wopr (v1.0) supports Intel, Motorola and Mostek Hexadecimal files\n\n\n"); 
	exit(0);
}

extern void loadhex(char *filename,int romen);

void wopr(int argc, char **argv)
{
	int i,r=0,base=0;
	char *p;
	
	if ((MEM=malloc(RAMEND+1))==NULL) exit(0);

	for (i=0;i<=ROMEND;i++) MEM[i]=0xff;
	for (;i<=RAMEND;i++) MEM[i]=0;
	
	/******** Parse Arguments *********/
	for (i=1;i<argc;i++) {
		if (*argv[i]=='-') {
			p=argv[i]; p++;
			switch (*p) {
			case 'b':
			case 'B':
				i++; if (i>=argc) usemsg();
				sscanf(argv[i],"%x",&base);
				i++; if (i>=argc) usemsg();
				loadmem(argv[i],base);
				printf("Binary File Loaded :%s at Base :0x%x\n",argv[i],base);
				break;
			case 'h':
			case 'H':
			default:
				usemsg();
			}
		} else loadhex(argv[i],r);
	}
	
	loadmem("wopr.rom",0);
	CPUREG.PC.W=0;
	Z80(CPUREG);
}

extern reg *Rpointer;

void woprreset()
{
	Rpointer->PC.W=0;
}

void Patch(reg *R){}
word Interrupt(reg *R){ return 0;}
